function [stt, st1, pttb , pt1b, wt, vt, ll, s00 , p00] = KALFIL(yt,xy,xs,A,H,R,G,F,Q,init,si00,pi00)  

%
%   Set up Kalman Filter
%
%
%   y(t) = Axy(t) + Hs(t) + w(t)   =  observation equation, xy(t) is exogenous, predetermined variables
%
%   s(t) = Gxs(t) + Fs(t) + v(t)   -  state equation
%
%
%
%   In the program below, will use the following notation
%
%
%  stt = s(t|t), st1 = s(t|t-1) 
%  ptt = p(t|t), pt1 = p(t|t-1)
%  kt = Kalman gain
%  s00 = s(0|0) - initial estimate of state vector
%  p00 = p(0|0) - initial estiamte of uncertainty
%  Q = E(vv') - var/cov matrix of noise in state equation
%  R = E(ww') - var/cov matrix of noise in observation equation 

% initialize Kalman filter
%
% set initial condition for the Kalman filter
%

if nargin<10; init=0; si00=0; pi00=0; end;  % If no initial value is given


n1 = size(F,1);
[nv nt] = size(yt);

st1 = zeros(n1,nt);
stt = zeros(n1,nt);
et = zeros(nv,nt);
ll = zeros(nt,1);

if init == 1
s00 = si00;
p00 = pi00;
else 
xsmean=mean(xs');
s00 = (eye(n1)-F)\G*xsmean'; % use unconditional distribution
clear xsmean;
%p00 = Q;
%for jj = 1:50
%p00 = p00 + F^jj*Q*(F')^jj;
%end
Q00 = reshape(Q,n1^2,1);
vecp00 = (eye(n1^2)- kron(F,F))\Q00;
clear Q00;
p00 = reshape(vecp00,n1,n1);
clear vecp00
end
st1(:,1) = G*xs(:,1) + F*s00;
pt1 = F*p00*F' + Q;
kt = pt1*H'/(H*pt1*H' + R);
stt(:,1) = (eye(n1)-kt*H)*st1(:,1) + kt*(yt(:,1)-A*xy(:,1)); 
ptt = pt1 - kt*H*pt1;
pt1b = pt1;
pttb = ptt;

wt(:,1) = yt(:,1)-A*xy(:,1)-H*st1(:,1);
ll(1,1) = -.5*log(det(H*pt1*H'+R))-.5*(wt(:,1)'/(H*pt1*H'+R)*wt(:,1)); % log likelihood

% run Kalman filter

for tt = 2:nt;
st1(:,tt) = G*xs(:,tt) + F*stt(:,tt-1); 
pt1 = F*ptt*F' + Q;
kt = pt1*H'/(H*pt1*H' + R);
stt(:,tt) = (eye(n1)-kt*H)*st1(:,tt) + kt*(yt(:,tt)-A*xy(:,tt)) ;
ptt = pt1 - kt*H*pt1;
pt1b = [pt1b ; pt1];
pttb = [pttb ; ptt];
wt(:,tt) = yt(:,tt)-A*xy(:,tt)-H*st1(:,tt);
ll(tt,1) = -.5*log(det(H*pt1*H'+R))-.5*(wt(:,tt)'/(H*pt1*H'+R)*wt(:,tt)); % log likelihood

end

vt = stt-st1;
