%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This is the main file which generates the basic results of the paper 
% "Measuring the world natural rate of interest" by Mark Wynne and Ren
% Zhang.
%
% Ren Zhang: renz@bgsu.edu
%
% June, 2017
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;
clc;
close all;
addpath('sims');        % Optimization algorithm by CA sims
 
percap=1;               % =0, baseline case
                        % =1, per capita data case 
                        % To run the code smoothly, 
                        

excel_ind=0; %1+percap;     % =0, do not write the results into excel
                            % =1 or 2, write all the results into excel.
            

data0=xlsread('dataagg160619.xlsx','data');
data59=xlsread('dataagg160619.xlsx','splice47');

oecdrec0=xlsread('dataagg160619.xlsx','oecdrecess');





if percap==1;
    pop=xlsread('dataagg160619.xlsx','popq');
    popt=sum(pop,2)/1000;
    data0(:,1)=data0(:,1)./popt;          % GDP per capita
end;


y0=rst_data4(data0,data59);

% 1948.1 - current
% Column 1: XGDP (1948.1-present)
% Column 2: PCNIA spliced with PCXFE (1948.3-present)
% Column 3: Inflation expectations (1960.1-present)
% Column 4: POIL spliced with IMPPETROL (1949.1-present)
% Column 5: PMO spliced with IMPXCOMP (1956.2-present)
% Column 6: NYFRB spliced with RFFE (1948.1-present)




begyear=1948;     begqtr=1;
endyear=2015;     endqtr=4;

begsampleyr=1961; begsampleqtr=1; 
endsampleyr=2015; endsampleqtr=4; % Select the sample range.

 
nstart=4*(begsampleyr-begyear)+begsampleqtr-begqtr;
T0=4*(endsampleyr-begsampleyr)+endsampleqtr-begsampleqtr+1;


y0(:,4)=y0(:,4)-y0(:,2);          % relative oil price inflation pi^o-pi
y0(:,5)=y0(:,5)-y0(:,2);          % relative import price inflation pi^i-pi
y0(:,6)=y0(:,6)-y0(:,3);          % Ex ante real rate 




%gy=400*(y0(nstart:nstart+T0-1,1)-y0(nstart-1:nstart+T0-2,1));




%********************First Stage***************************
[stTs2,lambda_z,param_s2,ll_s2]=rstar_s2(y0, T0,nstart);

lambda_z

%***********************Second Stage*************************

lam_g=param_s2(13);                       %Set starting value of the parameter lambda_g;

[stTs3,param_s3,param0_s3,ll_s3,hessian,grad]=rstar_s3(y0, T0,nstart,lam_g,lambda_z);
 
yps3=stTs3(1,:)';
yps3n1=stTs3(1,:)';
yps3n2=stTs3(2,:)';
gn1=stTs3(4,:)';
zn1=stTs3(6,:)';

gyps3=yps3n1(1:end,:)-yps3n2(1:end,:);


ygaps3=100*y0(nstart+1:nstart+T0,1)-yps3;

rstar=param_s3(8)*gn1+zn1;



%*************************************************************
dates=begsampleyr+0.25*begsampleqtr:0.25:endsampleyr+0.25*endsampleqtr;
dates=dates';
datesy=floor(dates);
datesm=(dates-datesy)*12+1;
datesd=ones(size(datesy));

dates2=datenum([datesy,datesm,datesd]);

oecdrec=oecdrecession(oecdrec0);      % OECD recession dates


figure(2)
    plot(dates2,ygaps3,'-b', 'LineWidth',1.6) ;
    hold on
    plot(dates2,zeros(T0,1),'--k')
   xlim([dates2(1)-366 dates2(end)])
    ylim([-6 6])
    xlabel('Year')
    ylabel('Output Gap')
    datetick('x','yyyy','keepticks')
    if excel_ind==1;
    xlswrite('worldrate',ygaps3,'figure2','B2')
    end;
recessionplot('recessions',oecdrec);   
 %set(gca,'Position',[0.1 0.5  0.8 0.4]);



% rstar=rstar/4;



%%%%%%%%%%%%%%%%%%%%%%%%%06/20/16%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    
    
      
    
  kinglow=xlsread('dataagg160619.xlsx','kinglowint');
  wrweight0=wrev(kinglow(:,1));
  wrunweight0=wrev(kinglow(:,2));  
    begklyr=1985; begklqtr=1;
    endklyr=2013; endklqtr=4;
    
   n1985q1= nan((-begsampleyr+begklyr)*4+(begklqtr-begsampleqtr),1);
    n2013q4=nan((endsampleyr-endklyr)*4+(endsampleqtr-endklqtr),1);
    wrweight=[n1985q1;wrweight0;n2013q4];
    wrunweight=[n1985q1;wrunweight0;n2013q4];

    %************************
    
figure (3)

plot(dates2,4*gn1,'-k','LineWidth',1.6); 
hold on
plot(dates2,4*gyps3,'--b','Linewidth',1.4);
 hold on
    plot(dates2,zeros(T0,1),'--k')
legend('Trend Growth','Potential Output Growth')

% hold on
% plot(dates,gy,'--r','Linewidth',1.2);
 ylim([-3 7])
 xlim([dates2(1)-365 dates2(end)]) 
%title('Trend Growth and Potential Output Growth, Baseline')
 xlabel('Year')
ylabel('Growth Rate, %');
    datetick('x','yyyy','keepticks')
recessionplot('recessions',oecdrec);   

%****************************
if excel_ind==1;
gyps3a=4*gyps3;
gn1a=4*gn1;
xlswrite('worldrate',gn1a,'figure3','B2')
xlswrite('worldrate',gyps3a,'figure3','C2')
end;
%*******************************

    
    

    
    
figure (4)
    
%subplot(2,1,1);

 
plot(dates,rstar,'-k','LineWidth',1.6); 
hold on
plot(dates,y0(nstart+1:nstart+T0,6),'--b','LineWidth',1.6)
  hold on
  
  plot(dates,wrweight,':r','LineWidth',1.6);
  hold on
  plot(dates,zeros(T0,1),'--k')
  
  if excel_ind==1;
 z000=zeros(T0,1);
 rrate=y0(nstart+1:nstart+T0,6);
 xlswrite('worldrate',rstar,'figure4','B2');
 xlswrite('worldrate',rrate,'figure4','C2');
 xlswrite('worldrate',wrweight,'figure4','D2');
 xlswrite('worldrate',z000,'figure4','E2');
  end;
 
 
legend('Natural Rate', 'Real Interest Rate','King-Low')
% ylim([-4 15])
   xlim([begsampleyr-1 endsampleyr+1]) 
%title('Real Interest Rate and Natural Rate of Interest, Baseline')
xlabel('Year')
ylabel('Natural Rate of Interest');

figure (5) 

 
plot(dates,rstar,'-k','LineWidth',1.6);
hold on
plot(dates,param_s3(8)*gn1,'--r','LineWidth',1.6)
hold on
plot(dates,zn1, ':b', 'LineWidth',2.6)
hold on
plot(dates,zeros(T0,1),'--k')
 
legend('Natural Rate', 'Trend Growth Component c*g', 'z Component')
   xlim([begsampleyr-1 endsampleyr+1]) 
%ylim([-4 14])
%title('Natural Rate of Interest and its Decomposition, Baseline')
xlabel('Year')
ylabel('Decomposition of Natural Rate of Interest');


  if excel_ind==1;
 z000=zeros(T0,1);
 gcont=param_s3(8)*gn1;
  xlswrite('worldrate',rstar,'figure5','B2');
 xlswrite('worldrate',gcont,'figure5','C2');
 xlswrite('worldrate',zn1,'figure5','D2');
 xlswrite('worldrate',z000,'figure5','E2');
  end;


rstarus=us15q4lw(excel_ind, percap);



if percap==0;
    rstarbase=rstar;
    ygaps3base=ygaps3;
    gn1base=gn1;
    save('rstarbase','rstarbase','ygaps3base','gn1base');
    
end;






if percap==1;
    
    load rstarbase;
    
    
    
    figure (8)
    plot(dates2,ygaps3,'-b', 'LineWidth',1.5) ;
    hold on
    
    plot(dates2,ygaps3base,':r', 'LineWidth',2.2) ;
    hold on
    plot(dates2,zeros(T0,1),'--k')
   xlim([dates2(1)-366 dates2(end)])
    ylim([-6 6])
    xlabel('Year')
    ylabel('Output Gap')
    datetick('x','yyyy','keepticks')
recessionplot('recessions',oecdrec);
    legend('Per Capita', 'Baseline')
    
        if excel_ind==1+percap;
  
    xlswrite('worldrate',ygaps3,'figure8','B2')
        xlswrite('worldrate',ygaps3base,'figure8','C2')
    xlswrite('worldrate',z000,'figure8','E2');
    end;
    

    
    figure (9)



 plot(dates2,4*gn1,'-k','LineWidth',1.6); 
hold on
plot(dates2,4*gyps3,'--b','Linewidth',1.4);
 hold on
 plot(dates2,4*gn1base,':r','LineWidth',1.6);
 hold on
    plot(dates2,zeros(T0,1),'--k')
legend('Trend Growth','Potential Output Growth', 'Trend Growth(Baseline)')
% hold on
% plot(dates,gy,'--r','Linewidth',1.2);
 ylim([-3 7])
 datetick('x','yyyy','keepticks')
 recessionplot('recessions',oecdrec);
 xlim([dates2(1)-366 dates2(end)]) 
 xlabel('Year')
 ylabel('Growth Rate, %')


    if excel_ind==1+percap;
        gn1a=4*gn1;
        gyps3a=4*gyps3;
        gn1basea=4*gn1base;
    xlswrite('worldrate',gn1a,'figure9','B2')
        xlswrite('worldrate',gn1basea,'figure9','C2')
    xlswrite('worldrate',gyps3a,'figure9','D2');
    xlswrite('worldrate',z000,'figure9','E2');
    end;
 
 
 

    
    
    figure (10)
    
%subplot(2,1,1);
 
plot(dates,rstar,'-k','LineWidth',1.6); 
hold on
plot(dates,y0(nstart+1:nstart+T0,6),'--b','LineWidth',1.6)
  hold on
  
  plot(dates,rstarbase,':r','LineWidth',1.6);
  hold on
  plot(dates,zeros(T0,1),'--k')
  
 
 
legend('Natural Rate (Per Capita)', 'Real Interest Rate','Natural Rate(Baseline)')
% ylim([-4 15])
   xlim([begsampleyr-1 endsampleyr+1]) 
%title('Real Interest Rate and Natural Rate of Interest, Baseline')
xlabel('Year')
ylabel('Natural Rate of Interest');    
  

    if excel_ind==1+percap;
    xlswrite('worldrate',rstar,'figure10','B2')
        xlswrite('worldrate',rstarbase,'figure10','C2')
rrate00=y0(nstart+1:nstart+T0,6);
    xlswrite('worldrate',rrate00,'figure10','D2');
    xlswrite('worldrate',z000,'figure10','E2');
    end;



end;
 




